package org.dasein.cloud.vsphere;

import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.RemoteException;

import org.dasein.cloud.AbstractCloud;
import org.dasein.cloud.CloudException;
import org.dasein.cloud.InternalException;
import org.dasein.cloud.services.address.AddressServices;
import org.dasein.cloud.services.dc.DataCenterServices;
import org.dasein.cloud.services.image.ImageServices;
import org.dasein.cloud.services.server.ServerServices;

import com.vmware.vim25.mo.ServiceInstance;

public class PrivateCloud extends AbstractCloud {
    public PrivateCloud() { }
    
    public AddressServices getAddressServices() {
        return new StaticIp(this);
    }
    
    @Override
    public String getCloudName() {
        return getContext().getCloudName();
    }

    @Override
    public String getProviderName() {
        return getContext().getProviderName();
    }
    
    @Override
    public DataCenterServices getDataCenterServices() {
        return new Dc(this);
    }
    
    private String getEndpoint(String regionId) {
        for( String endpoint : getEndpoints() ) {
            String rid = getRegionId(endpoint);
            
            if( rid.equals(regionId) ) {
                return endpoint;
            }
        }
        return null;
    }
    
    String[] getEndpoints() {
        String[] parts = getContext().getEndpoint().split(",");
        
        if( parts == null || parts.length < 1 ) {
            return new String[] { getContext().getEndpoint() };
        }
        return parts;
    }
    
    @Override
    public ImageServices getImageServices() {
        return new Template(this);
    }
    
    String getRegionId(String endpoint) {
        String[] parts = endpoint.split("/");
        
        if( parts == null || parts.length < 1 ) {
            return endpoint;
        }
        for( int i=0; i<parts.length; i++ ) {
            String part = parts[i];
            
            if( part == null ) {
                continue;
            }
            part = part.trim();
            if( part.length() < 1 ) {
                continue;
            }
            if( i == 0 && part.startsWith("http") ) {
                continue;
            }
            return part;
        }
        return endpoint;        
    }
    
    @Override 
    public ServerServices getServerServices() {
        return new Vm(this);
    }
    
    ServiceInstance getServiceInstance() throws CloudException, InternalException {
        try {
            String endpoint = getEndpoint(getContext().getRegionId());
            
            return new ServiceInstance(new URL(endpoint), new String(getContext().getAccessPublic(), "utf-8"), new String(getContext().getAccessPrivate(), "utf-8"), true);
        }
        catch( RemoteException e ) {
            throw new CloudException("Error creating service instance: " + e.getMessage());
        }
        catch( MalformedURLException e ) {
            throw new InternalException("Failed to generate endpoint URL for " + getContext().getEndpoint() + ": " + e.getMessage());
        }
        catch( UnsupportedEncodingException e ) {
            throw new InternalException("Encoding UTF-8 not supported: " + e.getMessage());
        }
    }
}
